# Source and build description

We include our software source in this directory.

- `afl-compiler-fuzzer` contains the source code for our modifications to AFL. Compile it by running `make` in the directory. This outputs the `afl-fuzz-compiler` binary. Invoke the `afl-fuzz-compiler` binary to see options. The upstream repository is maintained at https://github.com/agroce/afl-compiler-fuzzer

- `comby-mutation-server` contains the source code for running a mutation server powered by [comby](https://github.com/comby-tools/comby). The upstream repository is maintained at https://github.com/comby-tools/comby-mutation-server. This server is required when using our `afl-fuzz-compiler` binary with the `-2` option. Using the `-2` option calls out to the `comby-mutation-server` to get inputs to fuzz. To set up the server, just install the dependencies:

```
npm install express body-parser @iarna/toml minimist
```

and then run

```
$ export NODE_OPTIONS="--max-old-space-size=8192"
$ node server.js --generate 1.0
```

This will run the server on port `4448` by default. By default the server will
look for two directories to generate inputs from: `./fragments` and
`./templates`. The server will generate combinations of inputs by substituting
sources in `framents` into `templates`. Run `node server.js --help` to see
various options for running the server. See the project README for additional details.

# Evaluation, instrumentation, and fuzzing other targets

We evaluated our approach on 4 projects at the following commits:

- [`Solidity`](https://github.com/ethereum/solidity) @ `686b62b585d686f08fe2f8d586b8474c133dce2f`
- [`Move`](https://github.com/diem/diem) @ `bfb6b09715894b3c436919bf2e718b6ae0fcba9f`
- [`Fe`](https://github.com/ethereum/fe) @ `1ea2206e3d10e77163f1a01bee05088358d8ef23`
- [`Zig`](https://github.com/ziglang/zig) @ `2bd28a173cdb4c1205aea725416bb9c108a34619`

To fuzz these targets from source, please follow the build instructions in
each project's respective documentation. The binary targets we fuzzed for these projects are:

- `solidity/build/test/tools/solfuzzer` (Solidity)
- `diem/target/debug/move-build` (Move)
- `fe/target/debug/fe` (Fe)
- `zig/build-afl-llvm-mode/zig` (Zig)

Once building works, output binaries are instrumented by setting the `C` and `CXX` compiler flags for `Solidity and Zig` using those in `afl-compiler-fuzzer`. For example, building `Solidity` is done with:

```
cmake .. -DUSE_Z3=OFF -DCMAKE_C_COMPILER=/home/user/afl-compiler-fuzzer/afl-clang -DCMAKE_CXX_COMPILER=/home/user/afl-compiler-fuzzer/afl-clang++
AFL_INST_RATIO=10 make solfuzzer
```

For Rust-based projects (Move and Fe), and after building with `cargo build`, do the following. For example, for `Move`, go to `language/move-lang`:

Add

```
[profile.dev]
debug = true
panic = "abort"
```

to `Cargo.toml` so that the binary crashes on panic. Then:

```
cargo install afl --force
cargo afl build
rustup override set nightly
cargo afl build
```

The binary should now be instrumented successfully. Now, copy the `afl-compiler-fuzzer` to the directory where the Rust compiler is installed (this will depend on your system). You'll do something like:

```
cp /home/user/afl-fuzz-compiler /home/user/.local/share/afl.rs/rustc-1.46.0-04488af/afl.rs-0.8.0/afl/bin/afl-fuzz
```

Now it's possible to run `afl` by doing:


```
cargo afl <args> move-build @@
```

Or, alternatively invoke a command like:

```
/home/user/.local/share/afl.rs/rustc-1.49.0-e1884a8/afl.rs-0.8.0/afl/bin/afl-fuzz -1 0 -d -i move-programs -o move-crashes-1-00-trial-01 /home/rijnard/diem/target/debug/move-build @@
```

An exact and complete description of how we invoked each binary can be found in the `data` directory in the `fuzzer_stats` file for each trial, as well as associated inputs.
